
#include "static_if.hpp"
#include <functional>
#include <type_traits>
#include <iterator>
#include <vector>
#include <list>
#include <cassert>
#include <iostream>

template<typename Iter>
struct is_random_access_iterator : std::is_same<
    typename std::iterator_traits<Iter>::iterator_category,
    std::random_access_iterator_tag
> {};

template<typename Iter>
struct is_bidirectional_iterator : std::is_same<
    typename std::iterator_traits<Iter>::iterator_category,
    std::bidirectional_iterator_tag
> {};

template<typename Iter>
struct is_input_iterator : std::is_same<
    typename std::iterator_traits<Iter>::iterator_category,
    std::input_iterator_tag
> {};

template< typename Iter, typename Dist >
void myadvance ( Iter& i, Dist n ) {
    Iter* p = &i;
    static_if<is_random_access_iterator<Iter>::value>(
        std::bind([ ] ( auto p, auto n ) {
            *p += n;
        }, p, n)
    ).template elif<is_bidirectional_iterator<Iter>::value>(
        std::bind([ ] ( auto p, auto n ) {
            if(n >= 0) while(n--) ++*p;
            else while(n++) --*p;
        }, p, n)
    ).template elif<is_input_iterator<Iter>::value>(
        std::bind([ ] ( auto p, auto n ) {
            while(n--) ++p;
        }, p, n)
    ).else_(
        std::bind([ ] ( auto false_ ) {
            static_assert(false_, "requires at least InputIterator");
        }, std::false_type())
    );
}

int main ( ) {
    std::vector<char> v(3);
    v[0] = 'a'; v[1] = 'b'; v[2] = 'c';
    std::vector<char>::iterator r = v.begin(); // Random iterator.
    myadvance(r, 1);
    assert(*r == 'b');

    std::list<char> l(v.begin(), v.end());
    std::list<char>::iterator b = l.begin(); // Bidirectional, not random.
    myadvance(b, 2);
    assert(*b == 'c');

    return 0;
}

