
// Copyright (C) 2014 Lorenzo Caminiti.

#ifndef STATIC_IF_HPP_
#define STATIC_IF_HPP_

template< bool Condition >
struct static_if_statement;

template< bool Condition, typename Then >
static_if_statement<Condition> static_if ( Then const& branch ) {
    static_if_statement<Condition> if_;
    if_.then(branch);
    return if_;
}

template< bool Condition >
static_if_statement<Condition> static_if ( ) {
    return static_if_statement<Condition>();
}

template< bool Condition >
struct static_if_statement { // For `Condition = true`.
    void then ( ) {}

    template< typename Then >
    void then ( Then const& branch ) { branch(); }
    
    void else_ ( ) {}

    template< typename Else >
    void else_ ( Else const& ) {}
    
    template< bool ElifCondition >
    static_if_statement<true> elif ( ) { return static_if_statement<true>(); }

    template< bool ElifCondition, typename Elif >
    static_if_statement<true> elif ( Elif const& ) {
        return static_if_statement<true>();
    }
};

template< >
struct static_if_statement<false> { // For `Condition = false`.
    void then ( ) {}

    template< typename Then >
    void then ( Then const& ) {}
    
    void else_ ( ) {}

    template< typename Else >
    void else_ ( Else const& branch ) { branch(); }
    
    template< bool ElifCondition >
    static_if_statement<ElifCondition> elif ( ) {
        return static_if_statement<ElifCondition>();
    }

    template< bool ElifCondition, typename Elif >
    static_if_statement<ElifCondition> elif ( Elif const& branch ) {
        return static_if<ElifCondition>(branch);
    }
};

#endif // #include guard

